@@ -6,7 +6,7 @@ from .server import language_server
 logging.basicConfig(filename="galaxy-language-server.log", level=logging.DEBUG, filemode="w")
 
 
-def add_arguments(parser):
+def add_arguments(parser: argparse.ArgumentParser) -> None:
     parser.description = "Galaxy Language Server"
 
     parser.add_argument("--tcp", action="store_true", help="Use TCP server instead of stdio")
@@ -14,7 +14,7 @@ def add_arguments(parser):
     parser.add_argument("--port", type=int, default=2087, help="Bind to this port")
 
 
-def main():
+def main() -> None:
     parser = argparse.ArgumentParser()
     add_arguments(parser)
     args = parser.parse_args()
