@@ -1,15 +1,17 @@
 """This module provides a service to determine position context inside an XML document."""
 
-from typing import List, Optional
+from typing import List, Optional, cast
 
-from pygls.types import Range
-from pygls.workspace import Document, Position
+from pygls.lsp.types import Range
+from pygls.workspace import Position
 
-from .xml.constants import UNDEFINED_OFFSET, NEW_LINE
-from .xml.nodes import XmlSyntaxNode
-from .xml.parser import XmlDocumentParser
-from .xml.types import NodeType
-from .xsd.types import XsdNode, XsdTree
+from galaxyls.services.tools.constants import MACROS
+from galaxyls.services.xml.constants import UNDEFINED_OFFSET
+from galaxyls.services.xml.document import XmlDocument
+from galaxyls.services.xml.nodes import XmlElement, XmlSyntaxNode
+from galaxyls.services.xml.types import NodeType
+from galaxyls.services.xml.utils import convert_document_offsets_to_range
+from galaxyls.services.xsd.types import XsdNode, XsdTree
 
 
 class XmlContext:
@@ -21,20 +23,22 @@ class XmlContext:
 
     def __init__(
         self,
-        xsd_node: XsdNode,
+        xml_document: XmlDocument,
+        xsd_node: Optional[XsdNode],
         node: Optional[XmlSyntaxNode] = None,
         line_text: str = "",
         position: Optional[Position] = None,
         offset: int = UNDEFINED_OFFSET,
     ):
-        self._xsd_node: XsdNode = xsd_node
-        self._node: Optional[XmlSyntaxNode] = node
-        self._line_text: str = line_text
-        self._position: Optional[Position] = position
-        self._offset: int = offset
+        self.xml_document = xml_document
+        self._xsd_node = xsd_node
+        self._node = node
+        self._line_text = line_text
+        self._position = position
+        self._offset = offset
 
     @property
-    def token(self) -> Optional[XmlSyntaxNode]:
+    def node(self) -> Optional[XmlSyntaxNode]:
         """The syntax node at the context position."""
         return self._node
 
@@ -54,7 +58,17 @@ class XmlContext:
         return self._line_text
 
     @property
-    def xsd_element(self) -> XsdNode:
+    def characted_at_position(self) -> Optional[str]:
+        """The character at the context position."""
+        try:
+            if self._position:
+                return self._line_text[self._position.character]
+        except (IndexError, AttributeError):
+            pass
+        return None
+
+    @property
+    def xsd_element(self) -> Optional[XsdNode]:
         """The XSD element associated with the token in context."""
         return self._xsd_node
 
@@ -73,6 +87,19 @@ class XmlContext:
         """Indicates if the token in context is a tag."""
         return self._node is not None and self._node.node_type == NodeType.ELEMENT
 
+    @property
+    def is_tag_name(self) -> bool:
+        """Indicates if the token in context is a start tag name."""
+        return (
+            self.is_tag
+            and cast(XmlElement, self._node).name_start_offset <= self._offset <= cast(XmlElement, self._node).name_end_offset
+        )
+
+    @property
+    def is_attribute(self) -> bool:
+        """Indicates if the token in context is an attribute."""
+        return self._node is not None and self._node.is_attribute
+
     @property
     def is_attribute_key(self) -> bool:
         """Indicates if the token in context is an attribute key."""
@@ -83,6 +110,20 @@ class XmlContext:
         """Indicates if the token in context is an attribute value."""
         return self._node is not None and self._node.node_type == NodeType.ATTRIBUTE_VALUE
 
+    @property
+    def is_inside_attribute_value(self) -> bool:
+        """Indicates if the token in context is an attribute value."""
+        return (
+            self.is_attribute_value and self.node is not None and self.offset > self.node.start and self.offset < self.node.end
+        )
+
+    @property
+    def is_attribute_end(self) -> bool:
+        """Indicates that the context position is at the ending quote character of an attribute.
+
+        Example: <tag attribute="value["] <- The context position is at " """
+        return self.is_attribute_value and self._node is not None and self._offset == self._node.end - 1
+
     @property
     def attribute_name(self) -> Optional[str]:
         """The name of the attribute if the context is an attribute or None."""
@@ -91,15 +132,18 @@ class XmlContext:
     @property
     def is_content(self) -> bool:
         """Indicates if the token in context is within a content or CDATA block."""
-        return self._node is not None and (
-            self._node.node_type == NodeType.CONTENT or self._node.node_type == NodeType.CDATA_SECTION
-        )
+        return self._node is not None and self._node.node_type in [NodeType.CONTENT, NodeType.CDATA_SECTION]
 
     @property
     def is_closing_tag(self) -> bool:
         """Indicates if the token in context is a closing tag."""
         return self._node is not None and self._node.is_at_closing_tag(self._offset)
 
+    @property
+    def is_at_end(self) -> bool:
+        """Indicates if the context is at the last character of the node."""
+        return self._node is not None and self._node.end == self._offset
+
     @property
     def stack(self) -> List[str]:
         """The list of XML tag names from the root to the token in context."""
@@ -109,21 +153,28 @@ class XmlContext:
 
     def has_reached_max_occurs(self, node: XsdNode) -> bool:
         """Checks if the given node has reached the maximum number
-        of ocurrences.
+        of occurrences.
 
         Args:
             child (XsdNode): The node to check.
 
         Returns:
             bool: True if the node has reached the maximum number
-            of ocurrences permitted.
+            of occurrences permitted.
         """
         if node.max_occurs < 0:
             return False
-        target = self._node.parent or self._node
-        if target:
-            existing_count = sum(1 for child_node in target.children if child_node.name == node.name)
-            return existing_count >= node.max_occurs
+        if self._node:
+            target = self._node.parent or self._node
+            if target:
+                existing_count = sum(1 for child_node in target.children if child_node.name == node.name)
+                return existing_count >= node.max_occurs
+        return False
+
+    def is_valid_tag(self) -> bool:
+        """Indicates"""
+        if self.is_tag and self._xsd_node is not None and self._node is not None:
+            return self._node.name == self._xsd_node.name or self._node.name in [node.name for node in self._xsd_node.children]
         return False
 
 
@@ -135,7 +186,7 @@ class XmlContextService:
     def __init__(self, xsd_tree: XsdTree):
         self.xsd_tree = xsd_tree
 
-    def get_xml_context(self, document: Document, position: Position) -> XmlContext:
+    def get_xml_context(self, xml_document: XmlDocument, position: Position) -> XmlContext:
         """Gets the XML context at a given position inside the document.
 
         Args:
@@ -147,19 +198,17 @@ class XmlContextService:
             definition and other information. If the context can not be
             determined, the default context with no information is returned.
         """
-        offset = document.offset_at_position(position)
+        offset = xml_document.document.offset_at_position(position)
 
-        parser = XmlDocumentParser()
-        xml_document = parser.parse(document)
         if xml_document.is_empty:
-            return XmlContext(self.xsd_tree.root, node=None)
+            return XmlContext(xml_document, self.xsd_tree.root, node=None)
         node = xml_document.get_node_at(offset)
         xsd_node = self.find_matching_xsd_element(node, self.xsd_tree)
-        line_text = document.lines[position.line]
-        context = XmlContext(xsd_node, node, line_text, position, offset)
+        line_text = xml_document.document.lines[position.line]
+        context = XmlContext(xml_document, xsd_node, node, line_text, position, offset)
         return context
 
-    def find_matching_xsd_element(self, node: Optional[XmlSyntaxNode], xsd_tree: XsdTree) -> XsdNode:
+    def find_matching_xsd_element(self, node: Optional[XmlSyntaxNode], xsd_tree: XsdTree) -> Optional[XsdNode]:
         """Finds the xsd element in the XSD tree that matches the xml element associated with the given syntax node.
         If there is no matching node, the root (tool) xsd node is always returned.
 
@@ -168,22 +217,20 @@ class XmlContextService:
             xsd_tree (XsdTree): The XSD tree definition.
 
         Returns:
-            XsdNode: The matching xsd node.
+            XsdNode: The matching xsd node or None if there is no matching.
         """
         if node:
-            xsd_node = xsd_tree.find_node_by_stack(node.stack)
-            if xsd_node:
-                return xsd_node
-        return xsd_tree.root
+            node_stack = node.stack
+            if len(node_stack) > 0 and MACROS in node_stack:
+                return xsd_tree.find_node_by_name(node_stack[-1])
+            xsd_node = xsd_tree.find_node_by_stack(node_stack)
+            if xsd_node is None:
+                xsd_node = xsd_tree.find_node_by_stack(node_stack[:-1])
+            return xsd_node
+        return None
 
-    def get_range_for_context(self, document: Document, context: XmlContext) -> Range:
-        start_offset, end_offset = context.token.get_offsets(context.offset)
-        start_line = max(document.source.count(NEW_LINE, 0, start_offset), 0)
-        start_character = start_offset - document.source.rfind(NEW_LINE, 0, start_offset)
-        end_line = max(document.source.count(NEW_LINE, 0, end_offset), 0)
-        end_character = end_offset - document.source.rfind(NEW_LINE, 0, end_offset)
-
-        return Range(
-            Position(start_line, start_character),
-            Position(end_line, end_character),
-        )
+    def get_range_for_context(self, xml_document: XmlDocument, context: XmlContext) -> Range:
+        if context.node is None:
+            return Range()
+        start_offset, end_offset = context.node.get_offsets(context.offset)
+        return convert_document_offsets_to_range(xml_document.document, start_offset, end_offset)
